/*
Copyright (c) 2009, Bruno Golosio, Antonio Brunetti, Manuel Sanchez del Rio, Tom Schoonjans and Teemu Ikonen
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * The names of the contributors may not be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY Bruno Golosio, Antonio Brunetti, Manuel Sanchez del Rio, Tom Schoonjans and Teemu Ikonen ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Bruno Golosio, Antonio Brunetti, Manuel Sanchez del Rio, Tom Schoonjans and Teemu Ikonen BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef XRAYLIB_SHELLS_H
#define XRAYLIB_SHELLS_H

#define K_SHELL 0
#define L1_SHELL 1
#define L2_SHELL 2
#define L3_SHELL 3
#define M1_SHELL 4
#define M2_SHELL 5
#define M3_SHELL 6
#define M4_SHELL 7
#define M5_SHELL 8
#define N1_SHELL 9
#define N2_SHELL 10
#define N3_SHELL 11
#define N4_SHELL 12
#define N5_SHELL 13
#define N6_SHELL 14
#define N7_SHELL 15
#define O1_SHELL 16
#define O2_SHELL 17
#define O3_SHELL 18
#define O4_SHELL 19
#define O5_SHELL 20
#define O6_SHELL 21
#define O7_SHELL 22
#define P1_SHELL 23
#define P2_SHELL 24
#define P3_SHELL 25
#define P4_SHELL 26
#define P5_SHELL 27
#define Q1_SHELL 28
#define Q2_SHELL 29
#define Q3_SHELL 30

#endif
