/* Copyright (c) 2009-2013 Tom Schoonjans
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * The names of the contributors may not be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY Tom Schoonjans 'AS IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Tom Schoonjans BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef XRAYLIB_AUGER_H
#define XRAYLIB_AUGER_H

#define  K_L1L1_AUGER    0
#define  K_L1L2_AUGER    1
#define  K_L1L3_AUGER    2
#define  K_L1M1_AUGER    3
#define  K_L1M2_AUGER    4
#define  K_L1M3_AUGER    5
#define  K_L1M4_AUGER    6
#define  K_L1M5_AUGER    7
#define  K_L1N1_AUGER    8
#define  K_L1N2_AUGER    9
#define  K_L1N3_AUGER   10
#define  K_L1N4_AUGER   11
#define  K_L1N5_AUGER   12
#define  K_L1N6_AUGER   13
#define  K_L1N7_AUGER   14
#define  K_L1O1_AUGER   15
#define  K_L1O2_AUGER   16
#define  K_L1O3_AUGER   17
#define  K_L1O4_AUGER   18
#define  K_L1O5_AUGER   19
#define  K_L1O6_AUGER   20
#define  K_L1O7_AUGER   21
#define  K_L1P1_AUGER   22
#define  K_L1P2_AUGER   23
#define  K_L1P3_AUGER   24
#define  K_L1P4_AUGER   25
#define  K_L1P5_AUGER   26
#define  K_L1Q1_AUGER   27
#define  K_L1Q2_AUGER   28
#define  K_L1Q3_AUGER   29
#define  K_L2L1_AUGER   30
#define  K_L2L2_AUGER   31
#define  K_L2L3_AUGER   32
#define  K_L2M1_AUGER   33
#define  K_L2M2_AUGER   34
#define  K_L2M3_AUGER   35
#define  K_L2M4_AUGER   36
#define  K_L2M5_AUGER   37
#define  K_L2N1_AUGER   38
#define  K_L2N2_AUGER   39
#define  K_L2N3_AUGER   40
#define  K_L2N4_AUGER   41
#define  K_L2N5_AUGER   42
#define  K_L2N6_AUGER   43
#define  K_L2N7_AUGER   44
#define  K_L2O1_AUGER   45
#define  K_L2O2_AUGER   46
#define  K_L2O3_AUGER   47
#define  K_L2O4_AUGER   48
#define  K_L2O5_AUGER   49
#define  K_L2O6_AUGER   50
#define  K_L2O7_AUGER   51
#define  K_L2P1_AUGER   52
#define  K_L2P2_AUGER   53
#define  K_L2P3_AUGER   54
#define  K_L2P4_AUGER   55
#define  K_L2P5_AUGER   56
#define  K_L2Q1_AUGER   57
#define  K_L2Q2_AUGER   58
#define  K_L2Q3_AUGER   59
#define  K_L3L1_AUGER   60
#define  K_L3L2_AUGER   61
#define  K_L3L3_AUGER   62
#define  K_L3M1_AUGER   63
#define  K_L3M2_AUGER   64
#define  K_L3M3_AUGER   65
#define  K_L3M4_AUGER   66
#define  K_L3M5_AUGER   67
#define  K_L3N1_AUGER   68
#define  K_L3N2_AUGER   69
#define  K_L3N3_AUGER   70
#define  K_L3N4_AUGER   71
#define  K_L3N5_AUGER   72
#define  K_L3N6_AUGER   73
#define  K_L3N7_AUGER   74
#define  K_L3O1_AUGER   75
#define  K_L3O2_AUGER   76
#define  K_L3O3_AUGER   77
#define  K_L3O4_AUGER   78
#define  K_L3O5_AUGER   79
#define  K_L3O6_AUGER   80
#define  K_L3O7_AUGER   81
#define  K_L3P1_AUGER   82
#define  K_L3P2_AUGER   83
#define  K_L3P3_AUGER   84
#define  K_L3P4_AUGER   85
#define  K_L3P5_AUGER   86
#define  K_L3Q1_AUGER   87
#define  K_L3Q2_AUGER   88
#define  K_L3Q3_AUGER   89
#define  K_M1L1_AUGER   90
#define  K_M1L2_AUGER   91
#define  K_M1L3_AUGER   92
#define  K_M1M1_AUGER   93
#define  K_M1M2_AUGER   94
#define  K_M1M3_AUGER   95
#define  K_M1M4_AUGER   96
#define  K_M1M5_AUGER   97
#define  K_M1N1_AUGER   98
#define  K_M1N2_AUGER   99
#define  K_M1N3_AUGER  100
#define  K_M1N4_AUGER  101
#define  K_M1N5_AUGER  102
#define  K_M1N6_AUGER  103
#define  K_M1N7_AUGER  104
#define  K_M1O1_AUGER  105
#define  K_M1O2_AUGER  106
#define  K_M1O3_AUGER  107
#define  K_M1O4_AUGER  108
#define  K_M1O5_AUGER  109
#define  K_M1O6_AUGER  110
#define  K_M1O7_AUGER  111
#define  K_M1P1_AUGER  112
#define  K_M1P2_AUGER  113
#define  K_M1P3_AUGER  114
#define  K_M1P4_AUGER  115
#define  K_M1P5_AUGER  116
#define  K_M1Q1_AUGER  117
#define  K_M1Q2_AUGER  118
#define  K_M1Q3_AUGER  119
#define  K_M2L1_AUGER  120
#define  K_M2L2_AUGER  121
#define  K_M2L3_AUGER  122
#define  K_M2M1_AUGER  123
#define  K_M2M2_AUGER  124
#define  K_M2M3_AUGER  125
#define  K_M2M4_AUGER  126
#define  K_M2M5_AUGER  127
#define  K_M2N1_AUGER  128
#define  K_M2N2_AUGER  129
#define  K_M2N3_AUGER  130
#define  K_M2N4_AUGER  131
#define  K_M2N5_AUGER  132
#define  K_M2N6_AUGER  133
#define  K_M2N7_AUGER  134
#define  K_M2O1_AUGER  135
#define  K_M2O2_AUGER  136
#define  K_M2O3_AUGER  137
#define  K_M2O4_AUGER  138
#define  K_M2O5_AUGER  139
#define  K_M2O6_AUGER  140
#define  K_M2O7_AUGER  141
#define  K_M2P1_AUGER  142
#define  K_M2P2_AUGER  143
#define  K_M2P3_AUGER  144
#define  K_M2P4_AUGER  145
#define  K_M2P5_AUGER  146
#define  K_M2Q1_AUGER  147
#define  K_M2Q2_AUGER  148
#define  K_M2Q3_AUGER  149
#define  K_M3L1_AUGER  150
#define  K_M3L2_AUGER  151
#define  K_M3L3_AUGER  152
#define  K_M3M1_AUGER  153
#define  K_M3M2_AUGER  154
#define  K_M3M3_AUGER  155
#define  K_M3M4_AUGER  156
#define  K_M3M5_AUGER  157
#define  K_M3N1_AUGER  158
#define  K_M3N2_AUGER  159
#define  K_M3N3_AUGER  160
#define  K_M3N4_AUGER  161
#define  K_M3N5_AUGER  162
#define  K_M3N6_AUGER  163
#define  K_M3N7_AUGER  164
#define  K_M3O1_AUGER  165
#define  K_M3O2_AUGER  166
#define  K_M3O3_AUGER  167
#define  K_M3O4_AUGER  168
#define  K_M3O5_AUGER  169
#define  K_M3O6_AUGER  170
#define  K_M3O7_AUGER  171
#define  K_M3P1_AUGER  172
#define  K_M3P2_AUGER  173
#define  K_M3P3_AUGER  174
#define  K_M3P4_AUGER  175
#define  K_M3P5_AUGER  176
#define  K_M3Q1_AUGER  177
#define  K_M3Q2_AUGER  178
#define  K_M3Q3_AUGER  179
#define  K_M4L1_AUGER  180
#define  K_M4L2_AUGER  181
#define  K_M4L3_AUGER  182
#define  K_M4M1_AUGER  183
#define  K_M4M2_AUGER  184
#define  K_M4M3_AUGER  185
#define  K_M4M4_AUGER  186
#define  K_M4M5_AUGER  187
#define  K_M4N1_AUGER  188
#define  K_M4N2_AUGER  189
#define  K_M4N3_AUGER  190
#define  K_M4N4_AUGER  191
#define  K_M4N5_AUGER  192
#define  K_M4N6_AUGER  193
#define  K_M4N7_AUGER  194
#define  K_M4O1_AUGER  195
#define  K_M4O2_AUGER  196
#define  K_M4O3_AUGER  197
#define  K_M4O4_AUGER  198
#define  K_M4O5_AUGER  199
#define  K_M4O6_AUGER  200
#define  K_M4O7_AUGER  201
#define  K_M4P1_AUGER  202
#define  K_M4P2_AUGER  203
#define  K_M4P3_AUGER  204
#define  K_M4P4_AUGER  205
#define  K_M4P5_AUGER  206
#define  K_M4Q1_AUGER  207
#define  K_M4Q2_AUGER  208
#define  K_M4Q3_AUGER  209
#define  K_M5L1_AUGER  210
#define  K_M5L2_AUGER  211
#define  K_M5L3_AUGER  212
#define  K_M5M1_AUGER  213
#define  K_M5M2_AUGER  214
#define  K_M5M3_AUGER  215
#define  K_M5M4_AUGER  216
#define  K_M5M5_AUGER  217
#define  K_M5N1_AUGER  218
#define  K_M5N2_AUGER  219
#define  K_M5N3_AUGER  220
#define  K_M5N4_AUGER  221
#define  K_M5N5_AUGER  222
#define  K_M5N6_AUGER  223
#define  K_M5N7_AUGER  224
#define  K_M5O1_AUGER  225
#define  K_M5O2_AUGER  226
#define  K_M5O3_AUGER  227
#define  K_M5O4_AUGER  228
#define  K_M5O5_AUGER  229
#define  K_M5O6_AUGER  230
#define  K_M5O7_AUGER  231
#define  K_M5P1_AUGER  232
#define  K_M5P2_AUGER  233
#define  K_M5P3_AUGER  234
#define  K_M5P4_AUGER  235
#define  K_M5P5_AUGER  236
#define  K_M5Q1_AUGER  237
#define  K_M5Q2_AUGER  238
#define  K_M5Q3_AUGER  239
#define  L1_L2L2_AUGER  240
#define  L1_L2L3_AUGER  241
#define  L1_L2M1_AUGER  242
#define  L1_L2M2_AUGER  243
#define  L1_L2M3_AUGER  244
#define  L1_L2M4_AUGER  245
#define  L1_L2M5_AUGER  246
#define  L1_L2N1_AUGER  247
#define  L1_L2N2_AUGER  248
#define  L1_L2N3_AUGER  249
#define  L1_L2N4_AUGER  250
#define  L1_L2N5_AUGER  251
#define  L1_L2N6_AUGER  252
#define  L1_L2N7_AUGER  253
#define  L1_L2O1_AUGER  254
#define  L1_L2O2_AUGER  255
#define  L1_L2O3_AUGER  256
#define  L1_L2O4_AUGER  257
#define  L1_L2O5_AUGER  258
#define  L1_L2O6_AUGER  259
#define  L1_L2O7_AUGER  260
#define  L1_L2P1_AUGER  261
#define  L1_L2P2_AUGER  262
#define  L1_L2P3_AUGER  263
#define  L1_L2P4_AUGER  264
#define  L1_L2P5_AUGER  265
#define  L1_L2Q1_AUGER  266
#define  L1_L2Q2_AUGER  267
#define  L1_L2Q3_AUGER  268
#define  L1_L3L2_AUGER  269
#define  L1_L3L3_AUGER  270
#define  L1_L3M1_AUGER  271
#define  L1_L3M2_AUGER  272
#define  L1_L3M3_AUGER  273
#define  L1_L3M4_AUGER  274
#define  L1_L3M5_AUGER  275
#define  L1_L3N1_AUGER  276
#define  L1_L3N2_AUGER  277
#define  L1_L3N3_AUGER  278
#define  L1_L3N4_AUGER  279
#define  L1_L3N5_AUGER  280
#define  L1_L3N6_AUGER  281
#define  L1_L3N7_AUGER  282
#define  L1_L3O1_AUGER  283
#define  L1_L3O2_AUGER  284
#define  L1_L3O3_AUGER  285
#define  L1_L3O4_AUGER  286
#define  L1_L3O5_AUGER  287
#define  L1_L3O6_AUGER  288
#define  L1_L3O7_AUGER  289
#define  L1_L3P1_AUGER  290
#define  L1_L3P2_AUGER  291
#define  L1_L3P3_AUGER  292
#define  L1_L3P4_AUGER  293
#define  L1_L3P5_AUGER  294
#define  L1_L3Q1_AUGER  295
#define  L1_L3Q2_AUGER  296
#define  L1_L3Q3_AUGER  297
#define  L1_M1L2_AUGER  298
#define  L1_M1L3_AUGER  299
#define  L1_M1M1_AUGER  300
#define  L1_M1M2_AUGER  301
#define  L1_M1M3_AUGER  302
#define  L1_M1M4_AUGER  303
#define  L1_M1M5_AUGER  304
#define  L1_M1N1_AUGER  305
#define  L1_M1N2_AUGER  306
#define  L1_M1N3_AUGER  307
#define  L1_M1N4_AUGER  308
#define  L1_M1N5_AUGER  309
#define  L1_M1N6_AUGER  310
#define  L1_M1N7_AUGER  311
#define  L1_M1O1_AUGER  312
#define  L1_M1O2_AUGER  313
#define  L1_M1O3_AUGER  314
#define  L1_M1O4_AUGER  315
#define  L1_M1O5_AUGER  316
#define  L1_M1O6_AUGER  317
#define  L1_M1O7_AUGER  318
#define  L1_M1P1_AUGER  319
#define  L1_M1P2_AUGER  320
#define  L1_M1P3_AUGER  321
#define  L1_M1P4_AUGER  322
#define  L1_M1P5_AUGER  323
#define  L1_M1Q1_AUGER  324
#define  L1_M1Q2_AUGER  325
#define  L1_M1Q3_AUGER  326
#define  L1_M2L2_AUGER  327
#define  L1_M2L3_AUGER  328
#define  L1_M2M1_AUGER  329
#define  L1_M2M2_AUGER  330
#define  L1_M2M3_AUGER  331
#define  L1_M2M4_AUGER  332
#define  L1_M2M5_AUGER  333
#define  L1_M2N1_AUGER  334
#define  L1_M2N2_AUGER  335
#define  L1_M2N3_AUGER  336
#define  L1_M2N4_AUGER  337
#define  L1_M2N5_AUGER  338
#define  L1_M2N6_AUGER  339
#define  L1_M2N7_AUGER  340
#define  L1_M2O1_AUGER  341
#define  L1_M2O2_AUGER  342
#define  L1_M2O3_AUGER  343
#define  L1_M2O4_AUGER  344
#define  L1_M2O5_AUGER  345
#define  L1_M2O6_AUGER  346
#define  L1_M2O7_AUGER  347
#define  L1_M2P1_AUGER  348
#define  L1_M2P2_AUGER  349
#define  L1_M2P3_AUGER  350
#define  L1_M2P4_AUGER  351
#define  L1_M2P5_AUGER  352
#define  L1_M2Q1_AUGER  353
#define  L1_M2Q2_AUGER  354
#define  L1_M2Q3_AUGER  355
#define  L1_M3L2_AUGER  356
#define  L1_M3L3_AUGER  357
#define  L1_M3M1_AUGER  358
#define  L1_M3M2_AUGER  359
#define  L1_M3M3_AUGER  360
#define  L1_M3M4_AUGER  361
#define  L1_M3M5_AUGER  362
#define  L1_M3N1_AUGER  363
#define  L1_M3N2_AUGER  364
#define  L1_M3N3_AUGER  365
#define  L1_M3N4_AUGER  366
#define  L1_M3N5_AUGER  367
#define  L1_M3N6_AUGER  368
#define  L1_M3N7_AUGER  369
#define  L1_M3O1_AUGER  370
#define  L1_M3O2_AUGER  371
#define  L1_M3O3_AUGER  372
#define  L1_M3O4_AUGER  373
#define  L1_M3O5_AUGER  374
#define  L1_M3O6_AUGER  375
#define  L1_M3O7_AUGER  376
#define  L1_M3P1_AUGER  377
#define  L1_M3P2_AUGER  378
#define  L1_M3P3_AUGER  379
#define  L1_M3P4_AUGER  380
#define  L1_M3P5_AUGER  381
#define  L1_M3Q1_AUGER  382
#define  L1_M3Q2_AUGER  383
#define  L1_M3Q3_AUGER  384
#define  L1_M4L2_AUGER  385
#define  L1_M4L3_AUGER  386
#define  L1_M4M1_AUGER  387
#define  L1_M4M2_AUGER  388
#define  L1_M4M3_AUGER  389
#define  L1_M4M4_AUGER  390
#define  L1_M4M5_AUGER  391
#define  L1_M4N1_AUGER  392
#define  L1_M4N2_AUGER  393
#define  L1_M4N3_AUGER  394
#define  L1_M4N4_AUGER  395
#define  L1_M4N5_AUGER  396
#define  L1_M4N6_AUGER  397
#define  L1_M4N7_AUGER  398
#define  L1_M4O1_AUGER  399
#define  L1_M4O2_AUGER  400
#define  L1_M4O3_AUGER  401
#define  L1_M4O4_AUGER  402
#define  L1_M4O5_AUGER  403
#define  L1_M4O6_AUGER  404
#define  L1_M4O7_AUGER  405
#define  L1_M4P1_AUGER  406
#define  L1_M4P2_AUGER  407
#define  L1_M4P3_AUGER  408
#define  L1_M4P4_AUGER  409
#define  L1_M4P5_AUGER  410
#define  L1_M4Q1_AUGER  411
#define  L1_M4Q2_AUGER  412
#define  L1_M4Q3_AUGER  413
#define  L1_M5L2_AUGER  414
#define  L1_M5L3_AUGER  415
#define  L1_M5M1_AUGER  416
#define  L1_M5M2_AUGER  417
#define  L1_M5M3_AUGER  418
#define  L1_M5M4_AUGER  419
#define  L1_M5M5_AUGER  420
#define  L1_M5N1_AUGER  421
#define  L1_M5N2_AUGER  422
#define  L1_M5N3_AUGER  423
#define  L1_M5N4_AUGER  424
#define  L1_M5N5_AUGER  425
#define  L1_M5N6_AUGER  426
#define  L1_M5N7_AUGER  427
#define  L1_M5O1_AUGER  428
#define  L1_M5O2_AUGER  429
#define  L1_M5O3_AUGER  430
#define  L1_M5O4_AUGER  431
#define  L1_M5O5_AUGER  432
#define  L1_M5O6_AUGER  433
#define  L1_M5O7_AUGER  434
#define  L1_M5P1_AUGER  435
#define  L1_M5P2_AUGER  436
#define  L1_M5P3_AUGER  437
#define  L1_M5P4_AUGER  438
#define  L1_M5P5_AUGER  439
#define  L1_M5Q1_AUGER  440
#define  L1_M5Q2_AUGER  441
#define  L1_M5Q3_AUGER  442
#define  L2_L3L3_AUGER  443
#define  L2_L3M1_AUGER  444
#define  L2_L3M2_AUGER  445
#define  L2_L3M3_AUGER  446
#define  L2_L3M4_AUGER  447
#define  L2_L3M5_AUGER  448
#define  L2_L3N1_AUGER  449
#define  L2_L3N2_AUGER  450
#define  L2_L3N3_AUGER  451
#define  L2_L3N4_AUGER  452
#define  L2_L3N5_AUGER  453
#define  L2_L3N6_AUGER  454
#define  L2_L3N7_AUGER  455
#define  L2_L3O1_AUGER  456
#define  L2_L3O2_AUGER  457
#define  L2_L3O3_AUGER  458
#define  L2_L3O4_AUGER  459
#define  L2_L3O5_AUGER  460
#define  L2_L3O6_AUGER  461
#define  L2_L3O7_AUGER  462
#define  L2_L3P1_AUGER  463
#define  L2_L3P2_AUGER  464
#define  L2_L3P3_AUGER  465
#define  L2_L3P4_AUGER  466
#define  L2_L3P5_AUGER  467
#define  L2_L3Q1_AUGER  468
#define  L2_L3Q2_AUGER  469
#define  L2_L3Q3_AUGER  470
#define  L2_M1L3_AUGER  471
#define  L2_M1M1_AUGER  472
#define  L2_M1M2_AUGER  473
#define  L2_M1M3_AUGER  474
#define  L2_M1M4_AUGER  475
#define  L2_M1M5_AUGER  476
#define  L2_M1N1_AUGER  477
#define  L2_M1N2_AUGER  478
#define  L2_M1N3_AUGER  479
#define  L2_M1N4_AUGER  480
#define  L2_M1N5_AUGER  481
#define  L2_M1N6_AUGER  482
#define  L2_M1N7_AUGER  483
#define  L2_M1O1_AUGER  484
#define  L2_M1O2_AUGER  485
#define  L2_M1O3_AUGER  486
#define  L2_M1O4_AUGER  487
#define  L2_M1O5_AUGER  488
#define  L2_M1O6_AUGER  489
#define  L2_M1O7_AUGER  490
#define  L2_M1P1_AUGER  491
#define  L2_M1P2_AUGER  492
#define  L2_M1P3_AUGER  493
#define  L2_M1P4_AUGER  494
#define  L2_M1P5_AUGER  495
#define  L2_M1Q1_AUGER  496
#define  L2_M1Q2_AUGER  497
#define  L2_M1Q3_AUGER  498
#define  L2_M2L3_AUGER  499
#define  L2_M2M1_AUGER  500
#define  L2_M2M2_AUGER  501
#define  L2_M2M3_AUGER  502
#define  L2_M2M4_AUGER  503
#define  L2_M2M5_AUGER  504
#define  L2_M2N1_AUGER  505
#define  L2_M2N2_AUGER  506
#define  L2_M2N3_AUGER  507
#define  L2_M2N4_AUGER  508
#define  L2_M2N5_AUGER  509
#define  L2_M2N6_AUGER  510
#define  L2_M2N7_AUGER  511
#define  L2_M2O1_AUGER  512
#define  L2_M2O2_AUGER  513
#define  L2_M2O3_AUGER  514
#define  L2_M2O4_AUGER  515
#define  L2_M2O5_AUGER  516
#define  L2_M2O6_AUGER  517
#define  L2_M2O7_AUGER  518
#define  L2_M2P1_AUGER  519
#define  L2_M2P2_AUGER  520
#define  L2_M2P3_AUGER  521
#define  L2_M2P4_AUGER  522
#define  L2_M2P5_AUGER  523
#define  L2_M2Q1_AUGER  524
#define  L2_M2Q2_AUGER  525
#define  L2_M2Q3_AUGER  526
#define  L2_M3L3_AUGER  527
#define  L2_M3M1_AUGER  528
#define  L2_M3M2_AUGER  529
#define  L2_M3M3_AUGER  530
#define  L2_M3M4_AUGER  531
#define  L2_M3M5_AUGER  532
#define  L2_M3N1_AUGER  533
#define  L2_M3N2_AUGER  534
#define  L2_M3N3_AUGER  535
#define  L2_M3N4_AUGER  536
#define  L2_M3N5_AUGER  537
#define  L2_M3N6_AUGER  538
#define  L2_M3N7_AUGER  539
#define  L2_M3O1_AUGER  540
#define  L2_M3O2_AUGER  541
#define  L2_M3O3_AUGER  542
#define  L2_M3O4_AUGER  543
#define  L2_M3O5_AUGER  544
#define  L2_M3O6_AUGER  545
#define  L2_M3O7_AUGER  546
#define  L2_M3P1_AUGER  547
#define  L2_M3P2_AUGER  548
#define  L2_M3P3_AUGER  549
#define  L2_M3P4_AUGER  550
#define  L2_M3P5_AUGER  551
#define  L2_M3Q1_AUGER  552
#define  L2_M3Q2_AUGER  553
#define  L2_M3Q3_AUGER  554
#define  L2_M4L3_AUGER  555
#define  L2_M4M1_AUGER  556
#define  L2_M4M2_AUGER  557
#define  L2_M4M3_AUGER  558
#define  L2_M4M4_AUGER  559
#define  L2_M4M5_AUGER  560
#define  L2_M4N1_AUGER  561
#define  L2_M4N2_AUGER  562
#define  L2_M4N3_AUGER  563
#define  L2_M4N4_AUGER  564
#define  L2_M4N5_AUGER  565
#define  L2_M4N6_AUGER  566
#define  L2_M4N7_AUGER  567
#define  L2_M4O1_AUGER  568
#define  L2_M4O2_AUGER  569
#define  L2_M4O3_AUGER  570
#define  L2_M4O4_AUGER  571
#define  L2_M4O5_AUGER  572
#define  L2_M4O6_AUGER  573
#define  L2_M4O7_AUGER  574
#define  L2_M4P1_AUGER  575
#define  L2_M4P2_AUGER  576
#define  L2_M4P3_AUGER  577
#define  L2_M4P4_AUGER  578
#define  L2_M4P5_AUGER  579
#define  L2_M4Q1_AUGER  580
#define  L2_M4Q2_AUGER  581
#define  L2_M4Q3_AUGER  582
#define  L2_M5L3_AUGER  583
#define  L2_M5M1_AUGER  584
#define  L2_M5M2_AUGER  585
#define  L2_M5M3_AUGER  586
#define  L2_M5M4_AUGER  587
#define  L2_M5M5_AUGER  588
#define  L2_M5N1_AUGER  589
#define  L2_M5N2_AUGER  590
#define  L2_M5N3_AUGER  591
#define  L2_M5N4_AUGER  592
#define  L2_M5N5_AUGER  593
#define  L2_M5N6_AUGER  594
#define  L2_M5N7_AUGER  595
#define  L2_M5O1_AUGER  596
#define  L2_M5O2_AUGER  597
#define  L2_M5O3_AUGER  598
#define  L2_M5O4_AUGER  599
#define  L2_M5O5_AUGER  600
#define  L2_M5O6_AUGER  601
#define  L2_M5O7_AUGER  602
#define  L2_M5P1_AUGER  603
#define  L2_M5P2_AUGER  604
#define  L2_M5P3_AUGER  605
#define  L2_M5P4_AUGER  606
#define  L2_M5P5_AUGER  607
#define  L2_M5Q1_AUGER  608
#define  L2_M5Q2_AUGER  609
#define  L2_M5Q3_AUGER  610
#define  L3_M1M1_AUGER  611
#define  L3_M1M2_AUGER  612
#define  L3_M1M3_AUGER  613
#define  L3_M1M4_AUGER  614
#define  L3_M1M5_AUGER  615
#define  L3_M1N1_AUGER  616
#define  L3_M1N2_AUGER  617
#define  L3_M1N3_AUGER  618
#define  L3_M1N4_AUGER  619
#define  L3_M1N5_AUGER  620
#define  L3_M1N6_AUGER  621
#define  L3_M1N7_AUGER  622
#define  L3_M1O1_AUGER  623
#define  L3_M1O2_AUGER  624
#define  L3_M1O3_AUGER  625
#define  L3_M1O4_AUGER  626
#define  L3_M1O5_AUGER  627
#define  L3_M1O6_AUGER  628
#define  L3_M1O7_AUGER  629
#define  L3_M1P1_AUGER  630
#define  L3_M1P2_AUGER  631
#define  L3_M1P3_AUGER  632
#define  L3_M1P4_AUGER  633
#define  L3_M1P5_AUGER  634
#define  L3_M1Q1_AUGER  635
#define  L3_M1Q2_AUGER  636
#define  L3_M1Q3_AUGER  637
#define  L3_M2M1_AUGER  638
#define  L3_M2M2_AUGER  639
#define  L3_M2M3_AUGER  640
#define  L3_M2M4_AUGER  641
#define  L3_M2M5_AUGER  642
#define  L3_M2N1_AUGER  643
#define  L3_M2N2_AUGER  644
#define  L3_M2N3_AUGER  645
#define  L3_M2N4_AUGER  646
#define  L3_M2N5_AUGER  647
#define  L3_M2N6_AUGER  648
#define  L3_M2N7_AUGER  649
#define  L3_M2O1_AUGER  650
#define  L3_M2O2_AUGER  651
#define  L3_M2O3_AUGER  652
#define  L3_M2O4_AUGER  653
#define  L3_M2O5_AUGER  654
#define  L3_M2O6_AUGER  655
#define  L3_M2O7_AUGER  656
#define  L3_M2P1_AUGER  657
#define  L3_M2P2_AUGER  658
#define  L3_M2P3_AUGER  659
#define  L3_M2P4_AUGER  660
#define  L3_M2P5_AUGER  661
#define  L3_M2Q1_AUGER  662
#define  L3_M2Q2_AUGER  663
#define  L3_M2Q3_AUGER  664
#define  L3_M3M1_AUGER  665
#define  L3_M3M2_AUGER  666
#define  L3_M3M3_AUGER  667
#define  L3_M3M4_AUGER  668
#define  L3_M3M5_AUGER  669
#define  L3_M3N1_AUGER  670
#define  L3_M3N2_AUGER  671
#define  L3_M3N3_AUGER  672
#define  L3_M3N4_AUGER  673
#define  L3_M3N5_AUGER  674
#define  L3_M3N6_AUGER  675
#define  L3_M3N7_AUGER  676
#define  L3_M3O1_AUGER  677
#define  L3_M3O2_AUGER  678
#define  L3_M3O3_AUGER  679
#define  L3_M3O4_AUGER  680
#define  L3_M3O5_AUGER  681
#define  L3_M3O6_AUGER  682
#define  L3_M3O7_AUGER  683
#define  L3_M3P1_AUGER  684
#define  L3_M3P2_AUGER  685
#define  L3_M3P3_AUGER  686
#define  L3_M3P4_AUGER  687
#define  L3_M3P5_AUGER  688
#define  L3_M3Q1_AUGER  689
#define  L3_M3Q2_AUGER  690
#define  L3_M3Q3_AUGER  691
#define  L3_M4M1_AUGER  692
#define  L3_M4M2_AUGER  693
#define  L3_M4M3_AUGER  694
#define  L3_M4M4_AUGER  695
#define  L3_M4M5_AUGER  696
#define  L3_M4N1_AUGER  697
#define  L3_M4N2_AUGER  698
#define  L3_M4N3_AUGER  699
#define  L3_M4N4_AUGER  700
#define  L3_M4N5_AUGER  701
#define  L3_M4N6_AUGER  702
#define  L3_M4N7_AUGER  703
#define  L3_M4O1_AUGER  704
#define  L3_M4O2_AUGER  705
#define  L3_M4O3_AUGER  706
#define  L3_M4O4_AUGER  707
#define  L3_M4O5_AUGER  708
#define  L3_M4O6_AUGER  709
#define  L3_M4O7_AUGER  710
#define  L3_M4P1_AUGER  711
#define  L3_M4P2_AUGER  712
#define  L3_M4P3_AUGER  713
#define  L3_M4P4_AUGER  714
#define  L3_M4P5_AUGER  715
#define  L3_M4Q1_AUGER  716
#define  L3_M4Q2_AUGER  717
#define  L3_M4Q3_AUGER  718
#define  L3_M5M1_AUGER  719
#define  L3_M5M2_AUGER  720
#define  L3_M5M3_AUGER  721
#define  L3_M5M4_AUGER  722
#define  L3_M5M5_AUGER  723
#define  L3_M5N1_AUGER  724
#define  L3_M5N2_AUGER  725
#define  L3_M5N3_AUGER  726
#define  L3_M5N4_AUGER  727
#define  L3_M5N5_AUGER  728
#define  L3_M5N6_AUGER  729
#define  L3_M5N7_AUGER  730
#define  L3_M5O1_AUGER  731
#define  L3_M5O2_AUGER  732
#define  L3_M5O3_AUGER  733
#define  L3_M5O4_AUGER  734
#define  L3_M5O5_AUGER  735
#define  L3_M5O6_AUGER  736
#define  L3_M5O7_AUGER  737
#define  L3_M5P1_AUGER  738
#define  L3_M5P2_AUGER  739
#define  L3_M5P3_AUGER  740
#define  L3_M5P4_AUGER  741
#define  L3_M5P5_AUGER  742
#define  L3_M5Q1_AUGER  743
#define  L3_M5Q2_AUGER  744
#define  L3_M5Q3_AUGER  745
#define  M1_M2M2_AUGER  746
#define  M1_M2M3_AUGER  747
#define  M1_M2M4_AUGER  748
#define  M1_M2M5_AUGER  749
#define  M1_M2N1_AUGER  750
#define  M1_M2N2_AUGER  751
#define  M1_M2N3_AUGER  752
#define  M1_M2N4_AUGER  753
#define  M1_M2N5_AUGER  754
#define  M1_M2N6_AUGER  755
#define  M1_M2N7_AUGER  756
#define  M1_M2O1_AUGER  757
#define  M1_M2O2_AUGER  758
#define  M1_M2O3_AUGER  759
#define  M1_M2O4_AUGER  760
#define  M1_M2O5_AUGER  761
#define  M1_M2O6_AUGER  762
#define  M1_M2O7_AUGER  763
#define  M1_M2P1_AUGER  764
#define  M1_M2P2_AUGER  765
#define  M1_M2P3_AUGER  766
#define  M1_M2P4_AUGER  767
#define  M1_M2P5_AUGER  768
#define  M1_M2Q1_AUGER  769
#define  M1_M2Q2_AUGER  770
#define  M1_M2Q3_AUGER  771
#define  M1_M3M2_AUGER  772
#define  M1_M3M3_AUGER  773
#define  M1_M3M4_AUGER  774
#define  M1_M3M5_AUGER  775
#define  M1_M3N1_AUGER  776
#define  M1_M3N2_AUGER  777
#define  M1_M3N3_AUGER  778
#define  M1_M3N4_AUGER  779
#define  M1_M3N5_AUGER  780
#define  M1_M3N6_AUGER  781
#define  M1_M3N7_AUGER  782
#define  M1_M3O1_AUGER  783
#define  M1_M3O2_AUGER  784
#define  M1_M3O3_AUGER  785
#define  M1_M3O4_AUGER  786
#define  M1_M3O5_AUGER  787
#define  M1_M3O6_AUGER  788
#define  M1_M3O7_AUGER  789
#define  M1_M3P1_AUGER  790
#define  M1_M3P2_AUGER  791
#define  M1_M3P3_AUGER  792
#define  M1_M3P4_AUGER  793
#define  M1_M3P5_AUGER  794
#define  M1_M3Q1_AUGER  795
#define  M1_M3Q2_AUGER  796
#define  M1_M3Q3_AUGER  797
#define  M1_M4M2_AUGER  798
#define  M1_M4M3_AUGER  799
#define  M1_M4M4_AUGER  800
#define  M1_M4M5_AUGER  801
#define  M1_M4N1_AUGER  802
#define  M1_M4N2_AUGER  803
#define  M1_M4N3_AUGER  804
#define  M1_M4N4_AUGER  805
#define  M1_M4N5_AUGER  806
#define  M1_M4N6_AUGER  807
#define  M1_M4N7_AUGER  808
#define  M1_M4O1_AUGER  809
#define  M1_M4O2_AUGER  810
#define  M1_M4O3_AUGER  811
#define  M1_M4O4_AUGER  812
#define  M1_M4O5_AUGER  813
#define  M1_M4O6_AUGER  814
#define  M1_M4O7_AUGER  815
#define  M1_M4P1_AUGER  816
#define  M1_M4P2_AUGER  817
#define  M1_M4P3_AUGER  818
#define  M1_M4P4_AUGER  819
#define  M1_M4P5_AUGER  820
#define  M1_M4Q1_AUGER  821
#define  M1_M4Q2_AUGER  822
#define  M1_M4Q3_AUGER  823
#define  M1_M5M2_AUGER  824
#define  M1_M5M3_AUGER  825
#define  M1_M5M4_AUGER  826
#define  M1_M5M5_AUGER  827
#define  M1_M5N1_AUGER  828
#define  M1_M5N2_AUGER  829
#define  M1_M5N3_AUGER  830
#define  M1_M5N4_AUGER  831
#define  M1_M5N5_AUGER  832
#define  M1_M5N6_AUGER  833
#define  M1_M5N7_AUGER  834
#define  M1_M5O1_AUGER  835
#define  M1_M5O2_AUGER  836
#define  M1_M5O3_AUGER  837
#define  M1_M5O4_AUGER  838
#define  M1_M5O5_AUGER  839
#define  M1_M5O6_AUGER  840
#define  M1_M5O7_AUGER  841
#define  M1_M5P1_AUGER  842
#define  M1_M5P2_AUGER  843
#define  M1_M5P3_AUGER  844
#define  M1_M5P4_AUGER  845
#define  M1_M5P5_AUGER  846
#define  M1_M5Q1_AUGER  847
#define  M1_M5Q2_AUGER  848
#define  M1_M5Q3_AUGER  849
#define  M2_M3M3_AUGER  850
#define  M2_M3M4_AUGER  851
#define  M2_M3M5_AUGER  852
#define  M2_M3N1_AUGER  853
#define  M2_M3N2_AUGER  854
#define  M2_M3N3_AUGER  855
#define  M2_M3N4_AUGER  856
#define  M2_M3N5_AUGER  857
#define  M2_M3N6_AUGER  858
#define  M2_M3N7_AUGER  859
#define  M2_M3O1_AUGER  860
#define  M2_M3O2_AUGER  861
#define  M2_M3O3_AUGER  862
#define  M2_M3O4_AUGER  863
#define  M2_M3O5_AUGER  864
#define  M2_M3O6_AUGER  865
#define  M2_M3O7_AUGER  866
#define  M2_M3P1_AUGER  867
#define  M2_M3P2_AUGER  868
#define  M2_M3P3_AUGER  869
#define  M2_M3P4_AUGER  870
#define  M2_M3P5_AUGER  871
#define  M2_M3Q1_AUGER  872
#define  M2_M3Q2_AUGER  873
#define  M2_M3Q3_AUGER  874
#define  M2_M4M3_AUGER  875
#define  M2_M4M4_AUGER  876
#define  M2_M4M5_AUGER  877
#define  M2_M4N1_AUGER  878
#define  M2_M4N2_AUGER  879
#define  M2_M4N3_AUGER  880
#define  M2_M4N4_AUGER  881
#define  M2_M4N5_AUGER  882
#define  M2_M4N6_AUGER  883
#define  M2_M4N7_AUGER  884
#define  M2_M4O1_AUGER  885
#define  M2_M4O2_AUGER  886
#define  M2_M4O3_AUGER  887
#define  M2_M4O4_AUGER  888
#define  M2_M4O5_AUGER  889
#define  M2_M4O6_AUGER  890
#define  M2_M4O7_AUGER  891
#define  M2_M4P1_AUGER  892
#define  M2_M4P2_AUGER  893
#define  M2_M4P3_AUGER  894
#define  M2_M4P4_AUGER  895
#define  M2_M4P5_AUGER  896
#define  M2_M4Q1_AUGER  897
#define  M2_M4Q2_AUGER  898
#define  M2_M4Q3_AUGER  899
#define  M2_M5M3_AUGER  900
#define  M2_M5M4_AUGER  901
#define  M2_M5M5_AUGER  902
#define  M2_M5N1_AUGER  903
#define  M2_M5N2_AUGER  904
#define  M2_M5N3_AUGER  905
#define  M2_M5N4_AUGER  906
#define  M2_M5N5_AUGER  907
#define  M2_M5N6_AUGER  908
#define  M2_M5N7_AUGER  909
#define  M2_M5O1_AUGER  910
#define  M2_M5O2_AUGER  911
#define  M2_M5O3_AUGER  912
#define  M2_M5O4_AUGER  913
#define  M2_M5O5_AUGER  914
#define  M2_M5O6_AUGER  915
#define  M2_M5O7_AUGER  916
#define  M2_M5P1_AUGER  917
#define  M2_M5P2_AUGER  918
#define  M2_M5P3_AUGER  919
#define  M2_M5P4_AUGER  920
#define  M2_M5P5_AUGER  921
#define  M2_M5Q1_AUGER  922
#define  M2_M5Q2_AUGER  923
#define  M2_M5Q3_AUGER  924
#define  M3_M4M4_AUGER  925
#define  M3_M4M5_AUGER  926
#define  M3_M4N1_AUGER  927
#define  M3_M4N2_AUGER  928
#define  M3_M4N3_AUGER  929
#define  M3_M4N4_AUGER  930
#define  M3_M4N5_AUGER  931
#define  M3_M4N6_AUGER  932
#define  M3_M4N7_AUGER  933
#define  M3_M4O1_AUGER  934
#define  M3_M4O2_AUGER  935
#define  M3_M4O3_AUGER  936
#define  M3_M4O4_AUGER  937
#define  M3_M4O5_AUGER  938
#define  M3_M4O6_AUGER  939
#define  M3_M4O7_AUGER  940
#define  M3_M4P1_AUGER  941
#define  M3_M4P2_AUGER  942
#define  M3_M4P3_AUGER  943
#define  M3_M4P4_AUGER  944
#define  M3_M4P5_AUGER  945
#define  M3_M4Q1_AUGER  946
#define  M3_M4Q2_AUGER  947
#define  M3_M4Q3_AUGER  948
#define  M3_M5M4_AUGER  949
#define  M3_M5M5_AUGER  950
#define  M3_M5N1_AUGER  951
#define  M3_M5N2_AUGER  952
#define  M3_M5N3_AUGER  953
#define  M3_M5N4_AUGER  954
#define  M3_M5N5_AUGER  955
#define  M3_M5N6_AUGER  956
#define  M3_M5N7_AUGER  957
#define  M3_M5O1_AUGER  958
#define  M3_M5O2_AUGER  959
#define  M3_M5O3_AUGER  960
#define  M3_M5O4_AUGER  961
#define  M3_M5O5_AUGER  962
#define  M3_M5O6_AUGER  963
#define  M3_M5O7_AUGER  964
#define  M3_M5P1_AUGER  965
#define  M3_M5P2_AUGER  966
#define  M3_M5P3_AUGER  967
#define  M3_M5P4_AUGER  968
#define  M3_M5P5_AUGER  969
#define  M3_M5Q1_AUGER  970
#define  M3_M5Q2_AUGER  971
#define  M3_M5Q3_AUGER  972
#define  M4_M5M5_AUGER  973
#define  M4_M5N1_AUGER  974
#define  M4_M5N2_AUGER  975
#define  M4_M5N3_AUGER  976
#define  M4_M5N4_AUGER  977
#define  M4_M5N5_AUGER  978
#define  M4_M5N6_AUGER  979
#define  M4_M5N7_AUGER  980
#define  M4_M5O1_AUGER  981
#define  M4_M5O2_AUGER  982
#define  M4_M5O3_AUGER  983
#define  M4_M5O4_AUGER  984
#define  M4_M5O5_AUGER  985
#define  M4_M5O6_AUGER  986
#define  M4_M5O7_AUGER  987
#define  M4_M5P1_AUGER  988
#define  M4_M5P2_AUGER  989
#define  M4_M5P3_AUGER  990
#define  M4_M5P4_AUGER  991
#define  M4_M5P5_AUGER  992
#define  M4_M5Q1_AUGER  993
#define  M4_M5Q2_AUGER  994
#define  M4_M5Q3_AUGER  995

#endif
